/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

#if defined(ENV_MAPPING)
#define LIGHT_SPACE_WORLD_SPACE
#endif

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;
uniform lowp samplerCube s_TexUnit2; //!!!
uniform	lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;
#if defined(LIGHT_SPACE_WORLD_SPACE)
uniform	lowp vec3 u_sLightDirectionWS;
#define u_sLightDirection u_sLightDirectionWS
#else
uniform	lowp vec3 u_sLightDirection;
#endif

uniform	lowp vec3 u_dLightAmbient;
uniform	lowp vec3 u_dLightDiffuse;
#if defined(LIGHT_SPACE_WORLD_SPACE)
uniform	lowp vec3 u_dLightDirectionWS;
uniform	lowp vec3 u_dLightHalfDirectionWS;
#define u_dLightDirection u_dLightDirectionWS
#define u_dLightHalfDirection u_dLightHalfDirectionWS
#else
uniform	vec3 u_dLightDirection;
uniform	vec3 u_dLightHalfDirection;
#endif

#if defined(COLOR_MASK)
uniform lowp vec4 colorMask;
#endif


// Varyings

varying mediump vec2 v_TexCoord0;
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying lowp vec3 v_Color;
#endif

#if defined(ENV_MAPPING)
#if defined(NORMAL_MAPPING)
varying lowp vec3 v_ViewDir;
#else
varying lowp vec3 v_Reflection;
#endif
#endif

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
// doesn't work on tegra 2:
//varying lowp mat3 v_TSToLS;
#endif

void main()
{
    lowp vec4 texel = texture2D(s_TexUnit0, v_TexCoord0);
#if defined(ALPHA_TESTING)
    if (texel.a<=0.5) {
        discard;
    }
#endif

    // normal mapping
#if defined(NORMAL_MAPPING)
    lowp vec3 normal = normalize(texture2D(s_TexUnit1, v_TexCoord0).xyz*2.0 - 1.0);

    lowp mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;

    normal = TSToLS * normal;
#else
    lowp vec3 normal = v_Normal;
#endif

    // note: We scale all lighting effects by the albedo texture at the end
    lowp vec3 lighting = vec3(0,0,0);
#if defined(VERTEX_LIGHTING)
    lighting += v_Color;
#else
    lighting += u_dLightAmbient;
#endif

    // spot light diffuse
    lowp vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = max(dot(normal, sLightDir)*0.8 + 0.2, 0.0);
    lowp vec3 spotPower = vec3(spotNdotL*spotFactor);
    lighting += spotPower*u_sLightDiffuse;

    // directional light diffuse + specular
    float dLightScale = 0.0;
#if !defined(VERTEX_LIGHTING)
    dLightScale += max(dot(u_dLightDirection, normal), 0.0);
#endif
#if defined(SPECULAR_LIGHTING)
    dLightScale += pow(clamp(dot(normal, u_dLightHalfDirection), 0.0, 1.0), 70.0)*0.8;
#endif
	lowp vec3 dLightScaleVec = vec3(dLightScale);
    lighting += dLightScaleVec * u_dLightDiffuse;

#if defined(COLOR_MASK)
    lighting = mix(lighting.xyz, lighting.xyz*colorMask.xyz, texel.w);
#elif defined(ENV_MAPPING)
    lighting *= 0.81;
#endif

    // env map
#if defined(ENV_MAPPING)
#if defined(NORMAL_MAPPING)
    lowp vec3 reflectVec = -reflect(v_ViewDir, normal);
#else
    lowp vec3 reflectVec = v_Reflection;
#endif
    lowp vec3 cubeTex = textureCube(s_TexUnit2, reflectVec).xyz;
    float envScale = 0.3;
#if defined(COLOR_MASK)
    envScale = colorMask.a;
#endif
	lowp vec3 envScaleVec = vec3(envScale * u_sLightDiffuse.y);
    lighting += cubeTex * envScaleVec;
#endif

    gl_FragColor = vec4(lighting,1.0)*texel;
}
