/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;

uniform lowp vec3 u_sLightDirection;
#if defined(GROUND_TILE)
    uniform lowp vec3 u_sLightDiffuseScaled;
#else
    uniform lowp vec3 u_sLightDiffuse;
#endif
uniform vec2 u_sLightConeParams;

varying mediump vec2 v_TexCoord0;
varying mediump vec2 v_TexCoord1;
varying mediump vec3 v_Vertex;
varying lowp vec3 v_Color;
#if !defined(GROUND_TILE)
    varying lowp vec3 v_Normal;
#endif

void main()
{
    lowp vec3 t_Color = v_Color;

    lowp vec3 VL = normalize(v_Vertex);
    float spotCosAngle = dot(VL, u_sLightDirection);
    lowp vec3 spotFactor = vec3(clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0));
#if defined(GROUND_TILE)
    // assume a constant ndotl value
    lowp vec3 spotColor = u_sLightDiffuseScaled;
#else
    lowp vec3 spotColor = vec3(max(dot(v_Normal,VL)*0.8 + 0.2, 0.0));
    spotColor *= u_sLightDiffuse;
#endif
    t_Color.xyz+=spotColor*spotFactor;

    lowp vec3 texel = texture2D(s_TexUnit0, v_TexCoord0).xyz;
    lowp vec3 lightmap = texture2D(s_TexUnit1, v_TexCoord1).xyz;

    gl_FragColor.xyz = (texel*t_Color)*lightmap;
}
