/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

#if defined(ENV_MAPPING)
#define LIGHT_SPACE_WORLD_SPACE
#endif

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform mat4 u_mwMatrix;
uniform mat4 u_mvMatrix;
uniform mat4 u_mvpMatrix;
uniform lowp vec3 u_dLightAmbient;
uniform lowp vec3 u_dLightDiffuse;
#if defined(LIGHT_SPACE_WORLD_SPACE)
uniform	mediump vec4 u_sLightPositionWS;
uniform mediump vec3 u_dLightDirectionWS;
#define u_sLightPosition u_sLightPositionWS
#define u_dLightDirection u_dLightDirectionWS
#else
uniform	mediump vec4 u_sLightPosition;
uniform mediump vec3 u_dLightDirection;
#endif

// for env mapping
uniform	vec3 u_cameraPosWS;


#if defined(UV_SCROLL)
uniform vec4 uvOffset;
#endif

#if defined(COW_ANIM)
uniform vec4 animState;
#endif

attribute vec3 Position;
attribute vec4 Normal;
#if defined(NORMAL_MAPPING) || defined(COW_ANIM)
attribute vec4 Tangent;
#endif
attribute vec2 TexCoord0;


// Varyings

varying mediump vec2 v_TexCoord0;
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying lowp vec3 v_Color;
#endif

#if defined(ENV_MAPPING)
#if defined(NORMAL_MAPPING)
varying lowp vec3 v_ViewDir;
#else
varying lowp vec3 v_Reflection;
#endif
#endif

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
// doesn't work on tegra 2:
//varying lowp mat3 v_TSToLS;
#endif


#if defined(LIGHT_SPACE_WORLD_SPACE)
#define u_MSToLSMatrix u_mwMatrix
#else
#define u_MSToLSMatrix u_mvMatrix
#endif


#if defined(COW_ANIM)
vec3 getCowAnimPos(vec3 pos, vec4 masks, vec4 animState);
#endif

void main()
{
    v_TexCoord0 = TexCoord0;
#if defined(UV_SCROLL)
    v_TexCoord0 += uvOffset.xy;
#endif
    vec4 pos=vec4(Position,1.0);
	
#if defined(COW_ANIM)
    pos.xyz = getCowAnimPos(pos.xyz, Tangent, animState);
#endif
	
    gl_Position = u_mvpMatrix * pos;
    
    vec3 posLS = (u_MSToLSMatrix * pos).xyz;
    vec3 normalLS = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0.0)).xyz);
#if !defined(NORMAL_MAPPING)
	v_Normal = normalLS;
#endif
    v_sLightDir = u_sLightPosition.xyz - posLS;
    
#if defined(NORMAL_MAPPING)
    // use transpose of matrix by swapping multiplication (GLSL addresses rows, not columns)
    v_TSToLS1 = normalize((u_MSToLSMatrix * vec4(Tangent.xyz,0)).xyz);
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(Normal.xyz, Tangent.xyz)*Tangent.w,0)).xyz);
    v_TSToLS3 = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0)).xyz);
#endif

#if defined(VERTEX_LIGHTING)
    // directional light diffuse + ambient
    float ndotl = max(dot(normalLS, u_dLightDirection), 0.0);
    v_Color = (ndotl*u_dLightDiffuse) + u_dLightAmbient;
#endif

    // calculate viewing direction in world space (note: light space is world space if we do env mapping)
#if defined(ENV_MAPPING)
    vec3 viewDir = normalize(u_cameraPosWS - posLS);
#if !defined(NORMAL_MAPPING)
    // calc reflected angle for environment map
    v_Reflection=-reflect(viewDir,normalLS);
#else
    v_ViewDir = viewDir;
#endif
#endif
    
#ifdef SWITCH_XY
    // switch coords to landscape mode for iphone
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}

#if defined(COW_ANIM)
vec3 getCowAnimPos(vec3 pos, vec4 masks, vec4 animState)
{
    // ==============================================================
    //      Head Animation
    // ==============================================================
    vec3 mSourcePosition = pos;
	vec4 u_attribute0 = animState;
	
    float mAngleY = u_attribute0.y;
    float mAngleX = u_attribute0.x;
    vec3 mPositionDeformedY, mPositionDeformedX;
    float mCosAngleY, mSinAngleY, mCosAngleX, mSinAngleX;
    /*
        mSkinning.r - influences for the head
        mSkinning.g - influences for the ears
        mSkinning.b - influences for the tail
    */
    // unsigned to signed
	vec4 mSkinning = masks;
    
    vec3 mPivotHead = vec3( -0.154, 1.407, 1.035 );

    // Translate vertices into custom position
    vec3 mPositionOffset = mSourcePosition.xyz - mPivotHead; 

    // calculate sinus and cosinus for the Haed rotation
    //sincos( mAngleY, mSinAngleY, mCosAngleY );
    //sincos( mAngleX, mSinAngleX, mCosAngleX );
	mSinAngleY = sin(mAngleY);
	mCosAngleY = cos(mAngleY);
	mSinAngleX = sin(mAngleX);
	mCosAngleX = cos(mAngleX);

    // multiply vertices on Ry rotation matrix : x * Ry
    mPositionDeformedY.x = mPositionOffset.x * mCosAngleY - mPositionOffset.z * mSinAngleY;
    mPositionDeformedY.y = mPositionOffset.y;
    mPositionDeformedY.z = mPositionOffset.x * mSinAngleY  + mPositionOffset.z * mCosAngleY;

    // multiply previous transformation into Rx rotation matrix: x * Ry * Rx
    mPositionDeformedX.x = mPositionDeformedY.x;
    mPositionDeformedX.y = mPositionDeformedY.y * mCosAngleX - mPositionDeformedY.z * mSinAngleX;
    mPositionDeformedX.z = mPositionDeformedY.y * mSinAngleX + mPositionDeformedY.z * mCosAngleX;

    // Apply Skinning 
    vec3 mPositionResultHead = mix( mSourcePosition.xyz, mPositionDeformedX + mPivotHead, mSkinning.x );

    // ==============================================================
    //      Ears Animation
    // ==============================================================
    vec3 mPivotEars = vec3( -0.174, 1.712, 1.702 );
    vec3 mPositionEars, mPositionDeformedEars;
    
    // Ears Rotation Y
    float mAngleE = u_attribute0.z;
    if ( mSourcePosition.x < 0.0 ) {
        mAngleE*=-1.0;
    }
	float mCosAngleE, mSinAngleE;
    //sincos( mAngleE, mSinAngleE, mCosAngleE );
	mSinAngleE = sin(mAngleE);
	mCosAngleE = cos(mAngleE);

    mPositionDeformedEars = mPositionResultHead - mPivotEars;

    mPositionEars.x = mPositionDeformedEars.x * mCosAngleE - mPositionDeformedEars.z * mSinAngleE;
    mPositionEars.y = mPositionDeformedEars.y;
    mPositionEars.z = mPositionDeformedEars.x * mSinAngleE  + mPositionDeformedEars.z * mCosAngleE;

    vec3 mPositionResultEars = mix( mPositionResultHead, mPositionEars + mPivotEars, mSkinning.g );

    // ==============================================================
    //      Tail Animation
    // ==============================================================
    //vec3 mPositionTail = vec3( 0.4 * mSkinning.b * sin( u_attribute0.w + mSkinning.b ), 0.0 , 0.0 );
    vec3 mPositionTail = vec3( 0.4 * mSkinning.b * sin(u_attribute0.w * mSkinning.b), 0.0 , 0.0 );
    vec3 mPositionResultTail = mPositionResultEars + mPositionTail;
    
    return mPositionResultTail;
}

#endif
