/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

uniform mat4 u_mwMatrix;
uniform mat4 u_mvpMatrix;
uniform vec3 u_dLightAmbient;
uniform vec3 u_dLightDiffuse;
uniform vec3 u_dLightDirectionWS;

uniform vec3 u_sLightDiffuseScaled;
uniform vec4 u_sLightPositionWS;
uniform vec3 u_sLightDirectionWS;
uniform vec2 u_sLightConeParams;

attribute vec3 Position;
//attribute vec4 Normal;
//attribute vec4 Tangent;
attribute vec2 TexCoord0;

varying mediump vec2 v_TexCoord0;
varying lowp vec3 v_Color;

#if defined(ANIMATE)
    uniform vec4 animCrop;
#endif

void main()
{
    v_TexCoord0 = TexCoord0;   
    #if defined(ANIMATE)
        vec3 direction = vec3(1,0,0)*0.1;
        float offset = sin(animCrop.x + Position.x * Position.z) * Position.y;
        vec4 pos = vec4(Position.xyz + direction * offset, 1.0);
    #else
        vec4 pos = vec4(Position, 1.0);    
    #endif
    
    gl_Position = u_mvpMatrix * pos; 

	vec3 N = vec3(0,1,0);
	//vec3 N = normalize((u_mvMatrix * vec4(0,1,0,0.0)).xyz);

    vec3 VL = normalize(u_sLightPositionWS.xyz -(u_mwMatrix * pos).xyz);

    float spotCosAngle = dot(VL, u_sLightDirectionWS);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    //float ndotlSpot = max( dot(N,VL)*0.8 + 0.2, 0.0);
    v_Color = spotFactor*u_sLightDiffuseScaled; //(ndotlSpot*spotFactor)*u_sLightDiffuse;

    // directional diffuse lighting
    float ndotl = max( dot(N,u_dLightDirectionWS), 0.0);
    v_Color += (ndotl*u_dLightDiffuse) + u_dLightAmbient;

    // switch coords to landscape mode for iphone
#ifdef SWITCH_XY
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}
