/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

uniform mat4 u_mvMatrix;
uniform mat4 u_mvpMatrix;
uniform vec3 u_dLightAmbient;
uniform vec3 u_dLightDiffuse;
uniform	vec3 u_dLightDirection;
uniform	vec4 u_sLightPosition;

attribute vec3 Position;
attribute vec4 Normal;
attribute vec4 Tangent;
attribute vec2 TexCoord0;

varying mediump vec2 v_TexCoord0; // field texture
varying mediump vec2 v_TexCoord1; // weight map
varying mediump vec2 v_TexCoord2; // windrow texture
varying mediump vec3 v_Vertex;
varying lowp vec3 v_Color;

void main()
{
    v_TexCoord0 = TexCoord0.xy*2.0;
    v_TexCoord1 = vec2(TexCoord0.x, 1.0-TexCoord0.y);
    v_TexCoord2 = TexCoord0.xy*4.0;

    vec4 pos=vec4(Position,1.0);
    gl_Position = u_mvpMatrix * pos;

    vec3 viewNormal = normalize((u_mvMatrix * vec4(Normal.xyz,0.0)).xyz);
    v_Vertex = u_sLightPosition.xyz - vec3(u_mvMatrix * pos);

    // diffuse lighting
    float diff = max( dot(viewNormal, u_dLightDirection), 0.0);
    v_Color = (diff*u_dLightDiffuse) + u_dLightAmbient;

    // switch coords to landscape mode for iphone
#ifdef SWITCH_XY
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}
